<?PHP
/**
 * PHPOpenBiz
 *
 * LICENSE
 *
 * This source file is subject to the BSD License that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @package    BizSystem
 * @copyright  Copyright (c) 2005 through 2009, Rocky Swen
 * @license    http://www.phpopenbiz.org/jim/license     GPL  
 */
/**
 * Configuration class - Configuration management class that has help methods to get data from config.xml
 *
 * @author     Rocky Swen <rocky@phpopenbiz.org>
 * @version    2.3 2009-06-01 
 */
class Configuration
{
    /**
     * Array of XML Data
     *
     * @var array
     */
    private $m_XmlArr;
    /**
     * Associate Array of pre-configure database conections
     *
     * @var array
     */
    private $m_DatabaseInfo;
    /**
     * Read Config.xml into a internal array.
     */
    public function __construct ()
    {
        $xmlFile = BizSystem::GetXmlFileWithPath("Config");
        $this->m_XmlArr = &BizSystem::GetXmlArray($xmlFile);
    }
    /**
     * Returns the <DataSource> defined in Config.xml as an array.
     * Returned array is a 2D map.
     * (DBName1 => ["Name"], ["Driver"], ["Server"], ["DBName"], ["User"], {"Password"])
     * (DBName2 => ["Name"], ["Driver"], ["Server"], ["DBName"], ["User"], {"Password"])
     * (...)
     * If DBName is given, returns the record only related to the given DBName, otherwise returns all records
     *
     * @param string $DBName
     * @return array
     */
    public function GetDatabaseInfo ($DBName = null)
    {
        if ($DBName && $this->m_DatabaseInfo[$DBName])
            return $this->m_DatabaseInfo[$DBName];
        if (! $this->m_XmlArr["CONFIG"]["DATASOURCE"]) {
            $errmsg = BizSystem::GetMessage("ERROR", "SYS_ERROR_NODBINFO");
            trigger_error($errmsg, E_USER_ERROR);
        }
        $breakflag = false;
        foreach ($this->m_XmlArr["CONFIG"]["DATASOURCE"]["DATABASE"] as $db) {
            if ($this->m_XmlArr["CONFIG"]["DATASOURCE"]["DATABASE"]["ATTRIBUTES"]) {
                $db = $this->m_XmlArr["CONFIG"]["DATASOURCE"]["DATABASE"];
                $breakflag = true;
            }
            $tmp["Name"] = $db["ATTRIBUTES"]["NAME"];
            $tmp["Driver"] = $db["ATTRIBUTES"]["DRIVER"];
            $tmp["Server"] = $db["ATTRIBUTES"]["SERVER"];
            $tmp["DBName"] = $db["ATTRIBUTES"]["DBNAME"];
            $tmp["User"] = $db["ATTRIBUTES"]["USER"];
            $tmp["Password"] = $db["ATTRIBUTES"]["PASSWORD"];
            $this->m_DatabaseInfo[$tmp["Name"]] = $tmp;
            if ($breakflag)
                break;
        }
        if ($DBName && $this->m_DatabaseInfo[$DBName])
            return $this->m_DatabaseInfo[$DBName];
        if ($DBName && ! isset($this->m_DatabaseInfo[$DBName])) {
            $errmsg = BizSystem::GetMessage("ERROR", "BDO_ERROR_INVALID_DBNAME", array($DBName));
            trigger_error($errmsg, E_USER_ERROR);
        }
        if (! $DBName)
            return $tmp;
    }
}